<?php

class dbClass extends PDO {
		
		var $schedule_db	= "salestool";
		var $salestool_db	= "salestool";

		var $error_message;			//  If an error occurs in data retrieval this will contain the error message.
		
		
		
		
		function isValidKey( $api_key="" ) {
			
			$sql_query = "
					
					SELECT		COUNT(*)			AS api_matches
					
					FROM		webapi_users
						AS		users
					
					WHERE		users.api_key = :api_key;
					
					";
			
			$sql = $this->prepare( $sql_query );
			$variables = array( ':api_key'=>$api_key );
			$sql->execute( $variables );
			
			$is_valid_key = ( (int)$sql->fetchColumn() == 1 );
			
			return( $is_valid_key );
		}
		
		
		
		
		function getVesselList( ) {
			
			$sql_query = "
					
					SELECT 		vessels.vessel_name		AS vessel_name
					
					FROM		{$this->schedule_db}.ml_vessels
						AS		vessels
						
					WHERE 		vessels.vessel_end_operation >= NOW()
						
					ORDER BY	vessel_name;

					";
			
			$sql = $this->prepare( $sql_query );
			$sql->execute();
			
			$a_vessels = $sql->fetchAll( PDO::FETCH_OBJ );
			
			return( $a_vessels );		
			
		}
		
		
		
		function getLastTripDate( $vessel="" ) {
			
			//  If a vessel is defined, match either the name or the key.
			//
			$vessel_filter = ( $vessel == "" ) ? "1" : "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";
			
			$sql_query = "
					
					SELECT		trips.trip_start_date		AS trip_date
					
					FROM		{$this->schedule_db}.ml_trips
						AS		trips
						
					JOIN		{$this->schedule_db}.ml_vessels
						AS		vessels
						ON		vessels.vessel_key = trips.trip_vessel
						
					WHERE		{$vessel_filter}
			
					ORDER BY	trip_date DESC
					
					LIMIT 		1;
					
					";
			
			$sql = $this->prepare( $sql_query );
			$sql->execute( array( ':vessel'=>$vessel ));
			
			$last_trip_date = $sql->fetchColumn();
			
			return( $last_trip_date );			
		}
		
		
		
		
		// Get a list of the cabin classes in use on a given vessel between two dates.
		//
		function getActiveCabinClasses( $vessel="", $start_date="", $end_date="" ) {
			
			//  If a vessel is defined, match either the name or the key.
			//
			$vessel_filter = ( $vessel == "" ) ? "1" : "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";
			
			$sql_query = "

					SELECT			classes.class_key			AS class_key,
									classes.class_name			AS class_name, 
									classes.class_sort_order	AS class_sort_order

					FROM			{$this->schedule_db}.ml_trips
						AS			trips

					JOIN			{$this->schedule_db}.ml_classes
						AS			classes
						ON			classes.class_vessel = trips.trip_vessel
						
					JOIN			{$this->schedule_db}.ml_vessels
						AS			vessels
						ON			vessels.vessel_key = trips.trip_vessel
						
					WHERE			{$vessel_filter}
						AND			trips.trip_start_date >= :start_date
						AND			trips.trip_end_date <= :end_date
						
					GROUP BY 		classes.class_key
					
					ORDER BY		classes.class_sort_order;
			
				";
			
			$sql = $this->prepare( $sql_query );
			
			$sql->bindParam( ':vessel', $vessel );
			$sql->bindParam( ':start_date', $start_date );
			$sql->bindParam( ':end_date', $end_date );
			
#			$a_parameters = array( 	':vessel' => $vessel, 
#									':start_date' => $start_date, 
#									':end_date' => $end_date 
#									);
			
			try {
				$sql->execute();
			}
			catch ( PDOException $e ) {
				echo $e -> getMessage();
				exit();
			}
			
			$a_classes = $sql->fetchAll( PDO::FETCH_OBJ );
			
			return( $a_classes );
		}
		
		
		
		
		function getVesselTrips( $vessel="", $start_date="", $end_date="", $classes="", $currency="" ) {
			
			$start_date = ( $start_date == "" ) ? date( "Y-m-d") : date( "Y-m-d", strtotime( $start_date ));
			$end_date = ( $end_date == "" ) ? "2099-12-31" : date( "Y-m-d", strtotime( $end_date ));
			

			//  If a vessel is defined, match either the name or the key.
			//
			if ( $vessel == "" ) {
				$vessel = 1;
				$vessel_filter = ":vessel = :vessel";
			}
			else {
				$vessel_filter = "(( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel))";
			}
			
			$sql_query = "
					
					SELECT			vessels.vessel_name										AS vessel_name,
									trips.trip_code											AS trip_code,
									trips.trip_start_date									AS trip_start_date, 
									trips.trip_end_date										AS trip_end_date, 
									DATEDIFF( trips.trip_end_date, trips.trip_start_date ) 	AS trip_nights,
									itins.itin_name_short									AS trip_itinerary,
									itins.itin_name											AS trip_itinerary_long,
									itins.itin_embarkation_point							AS trip_embarkation_point,
									itins.itin_disembarkation_point							AS trip_disembarkation_point,
									'TBA'													AS trip_prices,
									trips.trip_availability									AS trip_availability,
									itins.itin_key											AS trip_itin_key
									
					FROM			{$this->schedule_db}.ml_trips
						AS			trips
						
					JOIN			{$this->schedule_db}.ml_vessels
						AS			vessels
						ON			vessels.vessel_key = trips.trip_vessel

					LEFT JOIN		{$this->salestool_db}.st_vessels
						AS 			st_vessels
						ON 			st_vessels.ves_name = vessels.vessel_name
						
					JOIN			{$this->schedule_db}.ml_itineraries
						AS			itins
						ON			itins.itin_key = trips.trip_itin
						
					WHERE			{$vessel_filter}
						AND 		trips.trip_start_date >= :start_date
						AND 		trips.trip_start_date >= vessels.vessel_start_operation
						AND 		trips.trip_start_date >= st_vessels.ves_first_published_date
						AND			trips.trip_start_date  <= :end_date
			
					ORDER BY		trip_start_date;
					
					";
			
			$sql = $this->prepare( $sql_query );
			
			$sql->execute( array( 	':vessel' => $vessel, 
									':start_date' => $start_date, 
									':end_date' => $end_date 
									));
			
			$a_trips = $sql->fetchAll( PDO::FETCH_OBJ );
			
			//  Sub query -- get the latest price start date where the price range starts before the beginning
			//  of the trip and ends after the beginning of the trip. 
			//
			$sub_query = "
				SELECT 		MAX( prices_dates.price_start_date )		AS price_start_date
				
				FROM		{$this->schedule_db}.ml_prices
					AS		prices_dates
					
				WHERE		prices_dates.price_start_date <= :trip_start_date
					AND		prices_dates.price_end_date >= :trip_start_date
					AND		prices_dates.price_itin = :itin_key
				";
			
			//  Now that we have the trips, for each trip, get a list of the prices for each cabin. 
			//

			$sql_query = "
			
				SELECT				prices.price_class					AS price_class, 
									classes.class_name					AS class_name,
									prices.price_amount					AS price_amount,
									currency.cur_code					AS price_currency
					
				FROM 				{$this->schedule_db}.ml_prices
					AS				prices
					
				LEFT JOIN			{$this->schedule_db}.ml_vessels
					AS				vessels
					ON				vessels.vessel_key = prices.price_vessel
					
				LEFT JOIN			{$this->schedule_db}.ml_classes
					AS				classes
					ON				classes.class_key = prices.price_class
					
				LEFT JOIN 			{$this->schedule_db}.currencies
					AS				currency
					ON				currency.cur_key = prices.price_currency
					
				WHERE				{$vessel_filter}
					AND				prices.price_itin = :itin_key
					AND				prices.price_start_date = ( $sub_query )
					AND				prices.price_active = 'Yes'
					
				ORDER BY			classes.class_sort_order; 
				";
			
			$sql = $this->prepare( $sql_query );
			
			$sql->bindParam( ":vessel", $vessel );
			$sql->bindParam( ":itin_key", $itin_key );
			$sql->bindParam( ":trip_start_date", $trip_start_date );
			$sql->bindParam( ":trip_end_date", $trip_end_date );
						
			foreach ( $a_trips as $trip ) {
				
				$vessel 			= $trip->vessel_name;
				$itin_key 			= $trip->trip_itin_key;
				$trip_start_date	= $trip->trip_start_date;
				$trip_end_date		= $trip->trip_end_date;
				
				$sql->execute();
				
				$trip->trip_prices = $sql->fetchAll( PDO::FETCH_OBJ );
				
				unset( $trip->trip_itin_key );
			}
			
			return( $a_trips );
		}
		
		
		
		function getTripAvailability( $trip_code ) {
			
			//  Sub query to count the number of available spaces.
			//
			$sql_count_spaces = "

				SELECT 			COUNT(*)
				
				FROM			{$this->schedule_db}.pax_si_belegung_details
					AS			beld
					
				JOIN			{$this->schedule_db}.pax_si_belegung
					AS			bel
					ON			(( bel.bel_ter_key = beld.beld_ter_key ) AND ( bel.bel_platz_nr = beld.beld_platz_nr ))
						
				WHERE			beld.beld_ter_key 	= ter.ter_key
					AND			beld.beld_buchungsstatus = 'available'
					AND			bel.bel_kabine_status = 'public'

				";
			
			//  Main query to get the list of trips and the number of spaces 
			//  available on each.
			//
			$sql_query = "
					
					SELECT		ter.ter_code							AS trip_code,
								ter.ter_schiff							AS trip_vessel,
								roud.roud_name							AS trip_itinerary,
								ter.ter_von								AS trip_start_date,
								ter.ter_bis								AS trip_end_date,
								DATEDIFF( ter.ter_bis, ter.ter_von )	AS trip_nights,
								ter.ter_preis_1_wert					AS trip_price_eur,
								
								( {$sql_count_spaces} ) 				AS trip_spaces_available,
									
								terd.terd_bemerkung_public				AS trip_notes
					
					FROM		{$this->schedule_db}.pax_si_termine
						AS		ter
					
					JOIN		{$this->schedule_db}.pax_si_termine_details
						AS		terd
						ON		terd.terd_key = ter.ter_key
						
					JOIN		{$this->schedule_db}.pax_si_routen_details
						AS		roud
						ON		roud.roud_key = ter.ter_rou_key
						
					WHERE		ter.ter_code = :trip_code;
					";
			
			$sql_vars = array( ":trip_code" => $trip_code );
			
			$sql = $this->prepare( $sql_query );
			$sql->execute( $sql_vars );
			
			$a_results = $sql->fetchAll( PDO::FETCH_OBJ );
			
			$availability = ( count( $a_results ) == 0 ) 
				? FALSE
				: $a_results[ 0 ];
			
			return( $availability );
		}
		
	}
	?>
